/*
   Egon Animator
   Copyright (C) 1997  Ulric Eriksson <ulric@edu.stockholm.se>

   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2, or (at your option)
   any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */

/*
 *      cmds.c
 *
 *      This rather bulky module contains all the functions that implement
 *      commands.  It also handles initialization of the interface to those
 *      functions.
 */

#include <stdio.h>
#include <string.h>
#include <stdlib.h>
#include <ctype.h>
#include <unistd.h>
#include <X11/Xatom.h>
#include <X11/Xlib.h>
#include <X11/Intrinsic.h>
#include <X11/Xaw/Label.h>
#include <X11/xpm.h>

#include "animator.h"
#include "egon.h"
#include "types.h"
#include "../siod/siod.h"
#include "../common/fonts.h"
#include "../xcommon/embed.h"
#include "../common/cmalloc.h"
#include "../xcommon/dialogs.h"

extern int input_warp_pointer;	/* from input.c */

typedef struct {
	char *name;
	 LISP(*function) ();
} s_fn_table;

extern s_fn_table fn_table[];

/* moving around */

static char *testlist[] = {	"testitem0",
	"testitem1", "testitem2", "testitem3", "testitem4", "testitem5",
	"testitem6", "testitem7", "testitem8", "testitem9"
};

static LISP listsel_test()
{
	int i = listsel(topLevel, "Choose an item", testlist, 10);
	if (i < 0)
		printf("User clicked Cancel\n");
	else
		printf("User chose %d\n", i);
	return NIL;
}

static LISP what_cursor_position()
{
	return NIL;
}

static LISP spawn(LISP command)
{
	char *argv[20], *p, cmd[1024];
	int argc = 0;

	strcpy(cmd, get_c_string(command));
	for (p = strtok(cmd, " \t\r\n");
		p && argc < 20;
		p = strtok(NULL, " \t\r\n")) {
		argv[argc++] = p;
	}
	argv[argc] = NULL;
	if (!fork()) {
		/* this is the child */
		execvp(argv[0], argv);
		exit(0);
	}
	return NIL;
}

/* Windows and buffers */

/*X
   static void load_buffer()

   Load a buffer from file.
   X */
static LISP load_buffer()
{
	static char path[1024], name[1024];
	char fn[1024];
	char fmt[80];
	ani_object *matrix;
	buffer *b;
	static int need_init = 1;

	if (need_init) {
		getcwd(path, 1024);
		need_init = 0;
	}
	strcpy(name, "");
	fn[0] = '\0';
	if (fsel_input(topLevel, path, name, patterns, fmt)) {
		sprintf(fn, "%s/%s", path, name);
#if 1	/* create new buffer and all that stuff */
		matrix = new_matrix();
		b = new_buffer(matrix, buffer_name(fn), fn);
#else	/* use the old buffer */
		b = w_list->buf;
		strcpy(b->name, buffer_name(fn));
		strcpy(b->path, fn);
#endif
		llpr("Loading");

		w_list->buf = b;
		activate_window(w_list);
	/* loadmatrix doesn't use the buffer argument in Egon,
	   the Scheme code is simply loaded and executed */
		if (loadmatrix(fn, b, fmt))
			llpr("New file");
	/* this will fail if load returns 1 because the string pool is full */

		b->change = FALSE;
		w_list->buf = b;
		pr_scr_flag = TRUE;
	}
	activate_window(w_list);
	return NIL;
}

/* Load using an external program

   1. Ask for the program to use
   2. Run the program and save output to file in /tmp
   3. Load the file using NULL format (i.e. ask for type)
*/
static LISP load_external()
{
	static int loaders = 0;
	static char *loadname[20], *loadprog[20];
	static int need_init = 1;
	char program[256], param[256], fn[80], cmd[256];
	ani_object *matrix;
	buffer *b;
	int i;

	if (need_init) {
		FILE *fp;
		char fnl[1024];
		char *p, *q, b[256];

		sprintf(fnl, "%s/egon/external.load", siaghome);
		if ((fp = fopen(fnl, "r")) == NULL) {
			llpr("Can't open header file");
			return NIL;
		}
		while (!feof(fp) && loaders < 20) {
			fgets(b, 250, fp);
			if ((p = strtok(b, ":")) && (q = strtok(NULL, "\n"))) {
				loadname[loaders] = cstrdup(p);
				loadprog[loaders] = cstrdup(q);
				loaders++;
			}
		}
		fclose(fp);
		need_init = 0;
	}
	program[0] = param[0] = '\0';
	i = listsel(topLevel, "External Program:", loadname, loaders);

	if (i >= 0 && ask_for_str("Parameters:", param)) {
		sprintf(fn, "/tmp/egon%ld", (long)getpid());
		sprintf(cmd, "%s %s > %s", loadprog[i], param, fn);
		if (system(cmd)) {
			llpr("External program failed");
			return NIL;
		}

		matrix = new_matrix();
		b = new_buffer(matrix, buffer_name(fn), fn);
		llpr("Loading");

		if (loadmatrix(fn, b, NULL))
			llpr("New file");

		b->change = FALSE;
		w_list->buf = b;
		pr_scr_flag = TRUE;
	}
	activate_window(w_list);
	return NIL;
}

static LISP
delete_window()
{
	if (!remove_window(w_list))
		llpr("Attempt to delete sole ordinary window");
	else
		pr_scr_flag = TRUE;
	activate_window(w_list);
	return NIL;
}

static LISP
delete_other_windows()
{
	while (remove_window(w_list->next));
	pr_scr_flag = TRUE;
	return NIL;
}

static LISP
split_window_vertically()
{
	if (!split_window(w_list))
		llpr("This window is too small to split");
	else
		pr_scr_flag = TRUE;
	return NIL;
}

static LISP
other_window()
{
	activate_window(w_list->next);
	pr_scr_flag = TRUE;
	return NIL;
}

/*X
   static void save_buffer()

   Save the buffer in the currently active window to file.
   X */
static LISP save_buffer()
{
	llpr("Saving");

	if (savematrix(w_list->buf->path, w_list->buf, NULL))
		errorbox(topLevel, "Couldn't save");
	else {
		w_list->buf->change = FALSE;
		pr_scr_flag = TRUE;
	}
	return NIL;
}

/*X
   static void save_buffer_as()

   Save the buffer in the currently active window to a named file.
   X */
static LISP save_buffer_as()
{
	static char path[1024], name[1024];
	char fn[1024];
	char fmt[80];
	char *p;
	static int need_init = 1;

	if (need_init) {
		getcwd(path, 1024);
		need_init = 0;
	}
	p = strrchr(w_list->buf->path, '/');
	if (p) strcpy(name, p+1);
	else strcpy(name, w_list->buf->path);
	fn[0] = '\0';
	if (fsel_input(topLevel, path, name, patterns, fmt)) {
		sprintf(fn, "%s/%s", path, name);
		llpr("Saving");
		if (savematrix(fn, w_list->buf, fmt))
			errorbox(topLevel, "Couldn't save");
		else {
			w_list->buf->change = FALSE;
			strcpy(w_list->buf->path, fn);
		}
		pr_scr_flag = TRUE;
	}
	return NIL;
}

/* Save using an external program

   1. Ask for the program to use
   2. Save to a file in /tmp using NULL format (i.e. ask for type)
   3. Run the program and read the file as input
*/
static LISP save_external()
{
	static int savers = 0;
	static char *savename[20], *saveprog[20];
	static int need_init = 1;
	char program[256], param[256], fn[80], cmd[256];
	int i;

	if (need_init) {
		FILE *fp;
		char fnl[1024];
		char *p, *q, b[256];

		sprintf(fnl, "%s/egon/external.save", siaghome);
		if ((fp = fopen(fnl, "r")) == NULL) {
			llpr("Can't open saver file");
			return NIL;
		}
		while (!feof(fp) && savers < 20) {
			fgets(b, 250, fp);
			if ((p = strtok(b, ":")) && (q = strtok(NULL, "\n"))) {
				savename[savers] = cstrdup(p);
				saveprog[savers] = cstrdup(q);
				savers++;
			}
		}
		fclose(fp);
		need_init = 0;
	}

	program[0] = param[0] = '\0';
	i = listsel(topLevel, "External Program:", savename, savers);

	if (i >= 0 && ask_for_str("Parameters:", param)) {
		llpr("Saving");

		sprintf(fn, "/tmp/egon%ld", (long)getpid());
		if (savematrix(fn, w_list->buf, NULL)) {
			errorbox(topLevel, "Couldn't save");
			return NIL;
		}
		sprintf(cmd, "%s %s < %s", program, param, fn);
		if (system(cmd)) {
			llpr("External program failed");
			return NIL;
		}
	}
	return NIL;
}

#if USE_COMPLETION
/*X
   static void complete_name(char *name)
   This function takes a partially completed buffer name
   and returns the first buffer name that matches it.
   X */
static int complete_name(name)
char *name;
{
	buffer *b;
	int len;

	b = w_list->buf;	/* start with the next buffer */
	do {
		b = b->next;
		if ((len = strlen(name)) == 0 
				|| !strncmp(b->name, name, len)) {
			strcpy(name, b->name);
			return FALSE;
		}
	} while (b != w_list->buf);
	return FALSE;
}
#endif

static LISP
switch_to_buffer()
{
	buffer *b;
	char *blist[100];
	int nblist = 0, n;

	b = w_list->buf;
	do {
		b = b->next;
		blist[nblist++] = b->name;
	} while (b != w_list->buf);
	if ((n = listsel(topLevel, "Change Buffer:", blist, nblist)) >= 0)
		w_list->buf = find_buffer_by_name(blist[n]);
	activate_window(w_list);
	return NIL;
}

static LISP
kill_buffer()
{
	buffer *b, *next_b;
	window *w;
	char *blist[100];
	int nblist = 0, n;

	b = w_list->buf;
	do {
		b = b->next;
		blist[nblist++] = b->name;
	} while (b != w_list->buf);
	if ((n = listsel(topLevel, "Kill Buffer:", blist, nblist)) >= 0) {
		if ((b = find_buffer_by_name(blist[n])) != NULL) {
			if (b != b->next) {
				next_b = free_buffer(b);
				w = w_list;
				do {
					if (w->buf == b)
						w->buf = next_b;
					w = w->next;
				} while (w != NULL && w != w_list);
				pr_scr_flag = TRUE;
			}
			else llpr("Couldn't kill last buffer");
		}
	}
	activate_window(w_list);
	return NIL;
}

static LISP
preview_block()
{
	preview();
	return NIL;
}

static LISP
print_block()
{
	int nr, nk;

	nr = 0;
	nk = 0;
	printer((char *)0, w_list->buf, 0, 0, 0, 0);
	return NIL;
}

static LISP
execute_interpreter_command(LISP intpr)
{
	char prompt[80];
	char b[256];
	char *intname = get_c_string(intpr);
	int intp = name2interpreter(intname);
	if (intp < 0) return NIL;
	sprintf(prompt, "%s command: ", intname);
	b[0] = '\0';
	if (ask_for_str(prompt, b))
		exec_expr(intp, b);
	return NIL;
}

static char *quit_buttons[] = {"Yes", "No", "Cancel"};

static LISP lalertbox(LISP prompt, LISP buttons)
{
	char *btext[10], *p = get_c_string(prompt);
	int bno = 0, n;

	while (bno < 10 && NNULLP(buttons)) {
		char *c = get_c_string(car(buttons));
		if (c) btext[bno] = cstrdup(c);
		else btext[bno] = cstrdup("button");
		buttons = cdr(buttons);
		bno++;
	}
	if (p == NULL) p = "prompt";
	n = alertbox(topLevel, p, btext, bno);
	while (bno) cfree(btext[--bno]);
	return flocons(n);
}

static LISP quit_egon()
{
	char prompt[256];
	buffer *b = b_list;
	do {
		if (b->change) {
			sprintf(prompt, "Save %s?", b->name);
			switch (alertbox(topLevel,
					prompt, quit_buttons, 3)) {
			case 0:
				savematrix(b->path, b, NULL);
				break;
			case 2:
				return NIL;
			default:
				break;
			}
		}
		b = b->next;
	} while (b != b_list);
	exit(0);
	return NIL;
}

static LISP print_version()
{
	llpr(VERSION);
	return NIL;
}

static LISP buffer_changed(LISP bname)
{
	buffer *buf;

	if (NULLP(bname)) buf = w_list->buf;
	else buf = find_buffer_by_name(get_c_string(bname));

	if (buf) buf->change = TRUE;
	return NIL;
}

static LISP set_format(LISP format)
{
	ins_format(w_list->buf, w_list->object, w_list->script,
			get_c_long(format));
	return NIL;
}

static LISP get_format()
{
	long fmt = ret_format(w_list->buf, w_list->object, w_list->script);
	return flocons(fmt);
}

/* These functions allow implementation of commands in Scheme
	rather than in C with Scheme wrappers */

static LISP bell()
{
	XBell(display, 100);
	return NIL;
}

static LISP lask_for_str(LISP prompt, LISP buf)
{
	char *p, b[256];

	strcpy(b, get_c_string(buf));
	p = get_c_string(prompt);
	if (ask_for_str(p, b))
		return strcons(strlen(b), b);
	else
		return NIL;
}

static LISP lexecute(LISP cmd)
{
#ifdef DEBUG
	fprintf(stderr, "lexecute()\n");
#endif
	execute(get_c_string(cmd));
	return NIL;
}

static LISP linput_warp_pointer(LISP value)
{
	input_warp_pointer = get_c_long(value);
	return NIL;
}

/* Beginning of animation functions */
#if 1
#include "noname.c"

static LISP ani_c_test()
{
	ani_show(noname_objects, 0L, noname_delta, noname_duration,
		noname_width, noname_height, noname_background, NULL);
	return NIL;
}
#else
static LISP ani_c_test()
{
	return NIL;
}
#endif

static LISP ani_begin()
{
	buffer *b = w_list->buf;
	/* should free old objects first */
	b->cast = NULL;
	b->delta = 100;
	b->duration = 1000;
	b->width = 600;
	b->height = 400;
	b->bg = NULL;
	b->change = TRUE;
	w_list->object = NULL;
	w_list->script = NULL;
	pr_scr_flag = TRUE;
	return NIL;
}

static LISP ani_property(LISP name, LISP value)
{
	buffer *b = w_list->buf;
	ani_script *lasts = w_list->script;
	int n = get_c_long(name);

	if (!lasts) err("Last script is NULL", NIL);

	if (FLONUMP(value)) {
		long lv = get_c_long(value);
		switch (n) {
		case ANI_X:
			lasts->x = lv;
			break;
		case ANI_Y:
			lasts->y = lv;
			break;
		case ANI_WIDTH:
			lasts->width = lv;
			break;
		case ANI_HEIGHT:
			lasts->height = lv;
			break;
		case ANI_VISIBLE:
			lasts->visible = lv;
			break;
		case ANI_COLOR:
			lasts->color = lv;
			break;
		case ANI_FONT:
			lasts->font = lv;
			break;
		default:
			err("No such property", name);
		}
	} else {
		char *tv = get_c_string(value);
		switch (n) {
		case ANI_TEXT:
			lasts->string = cstrdup(tv);
			break;
		default:
			err("No such property", name);
		}
	}
	b->change = TRUE;
	pr_scr_flag = TRUE;
	return NIL;
}

static char *typenames[] = {
"None", "Line", "Rectangle", "Arc", "Ellipse", "Pixmap", "String",
"Point", "FilledRectangle", "FilledArc", "FilledEllipse", "Object"};

static char *type2name(buffer *buf, int type)
{
	char p[256], *t;
	ani_object *o = buf->cast;
	int n = 0;

	while (o) {
		if (o->type == type) n++;
		o = o->next;
	}
	if (type < ANI_NONE || type > ANI_FILLELLIPSE) t = "Object";
	else t = typenames[type];

	sprintf(p, "%s%d", t, n);
	return cstrdup(p);
}

static LISP select_object(LISP name)
{
	char *string = get_c_string(name);

	ani_object *o = w_list->buf->cast;

	while (o) {
		if (!strcmp(o->name, string)) break;
		o = o->next;
	}
	if (!o) {
		llpr("No such object");
	} else {
		w_list->object = o;
		w_list->script = o->script;
	}
	pr_scr_flag = TRUE;
	return NIL;
}

static LISP select_tick(LISP tick)
{
	ani_script *s;
	long t = get_c_long(tick);
	if (!w_list->object) {
		llpr("No objects!");
		return NIL;
	}
	s = w_list->object->script;
	while (s) {
		if (s->time == t) break;
		s = s->next;
	}
	if (!s) {
		llpr("No such tick");
	} else {
		w_list->script = s;
	}
	pr_scr_flag = TRUE;
	return NIL;
}

static LISP lani_object(LISP obj)
{
	buffer *b = w_list->buf;
	ani_object *lasto = w_list->object, *newo;
	ani_script *lasts;
	int t = get_c_long(obj);

	newo = (ani_object *)cmalloc(sizeof(ani_object));
	if (lasto) {	/* put newo after lasto */
		newo->next = lasto->next;
		lasto->next = newo;
	} else {	/* put newo first in object list */
		b->cast = newo;
		newo->next = NULL;
	}
	w_list->object = newo;
	newo->type = t;
	newo->name = type2name(b, t);
	lasts = newo->script = (ani_script *)cmalloc(sizeof(ani_script));
	lasts->x = 0;
	lasts->y = 0;
	lasts->width = 0;
	lasts->height = 0;
	lasts->visible = 1;
	lasts->time = 0;
	lasts->color = COLOR_0;
	lasts->font = HELVETICA | BOLD | SIZE_14;
	lasts->string = NULL;
	lasts->next = NULL;
	w_list->script = lasts;
	b->change = TRUE;
	pr_scr_flag = TRUE;
	return NIL;
}

static LISP set_type(LISP type)
{
	int t = get_c_long(type);

	if (!w_list->object) {
		llpr("No object selected");
	} else {
		w_list->object->type = t;
		cfree(w_list->object->name);
		w_list->object->name = type2name(w_list->buf, t);
	}
	w_list->buf->change = TRUE;
	pr_scr_flag = TRUE;
	return NIL;
}

/* FIXME: these two should free the memory used */
static LISP del_object()
{
	buffer *b = w_list->buf;
	ani_object *o = w_list->object;
	ani_object *o1 = w_list->buf->cast;

	if (!o) {
		llpr("None selected");
		return NIL;
	}
	if (o == o1) {	/* first object */
		w_list->buf->cast = o->next;
		w_list->object = o->next;
	} else {	/* some other object; find the one before */
		while (o1->next != o) o1 = o1->next;
		o1->next = o->next;
		w_list->object = o1;	/* select previous object */
	}
	if (w_list->object)
		w_list->script = w_list->object->script;
	else
		w_list->script = NULL;
	b->change = TRUE;
	pr_scr_flag = TRUE;
	return NIL;
}

static LISP del_time()
{
	buffer *b = w_list->buf;
	ani_object *o = w_list->object;
	ani_script *s, *s1;

	if (!o) {
		llpr("None selected");
		return NIL;
	}
	s = w_list->script;
	s1 = o->script;
	if (s->time == 0) {
		llpr("Can't delete tick 0");
		return NIL;
	}
	while (s1->next != s) s1 = s1->next;
	s1->next = s->next;
	w_list->script = s1;
	b->change = TRUE;
	pr_scr_flag = TRUE;
	return NIL;
}

static LISP ani_background(LISP bgp)
{
	buffer *b = w_list->buf;
	if (b->bg) cfree(b->bg);
	b->bg = cstrdup(get_c_string(bgp));
	b->change = TRUE;
	pr_scr_flag = TRUE;
	return NIL;
}

static LISP ani_time(LISP tim)
{
	ani_object *o = w_list->object;
	ani_script *s, *news;
	unsigned long t = get_c_long(tim);
	
	if (!o) return NIL;

	/* find the place to put new tick after */
	s = o->script;
	while (s->next && s->next->time <= t)
		s = s->next;

	if (s->time == t) return NIL;	/* dup tick */

	/* simply copy the last tick, but with a new time */
	news = (ani_script *)cmalloc(sizeof(ani_script));
	*news = *s;
	news->next = s->next;
	news->time = t;
	s->next = news;
	w_list->script = news;
	pr_scr_flag = TRUE;
	return NIL;
}

static LISP lani_duration(LISP tim)
{
	buffer *b = w_list->buf;
	b->duration = get_c_long(tim);
	pr_scr_flag = TRUE;
	return NIL;
}

static LISP lani_delta(LISP tim)
{
	buffer *b = w_list->buf;
	b->delta = get_c_long(tim);
	pr_scr_flag = TRUE;
	return NIL;
}

static LISP lani_geometry(LISP w, LISP h)
{
	buffer *b = w_list->buf;
	b->width = get_c_long(w);
	b->height = get_c_long(h);
	pr_scr_flag = TRUE;
	return NIL;
}

static LISP ani_end()
{
	buffer *b = w_list->buf;

	ani_show(b->cast, 0L, b->delta, b->duration, b->width, b->height,
		b->bg, b->path);
	return NIL;
}

static LISP lani_ctl(LISP mode, LISP now)
{
	int m = get_c_long(mode);
	long n = get_c_long(now);

	if (ani_ctl(m, n) == ANI_ERR)
		/* perhaps it wasn't popped up? */
		if (m == ANI_CONTINUE) return ani_end();
	return NIL;
}

/* Set up the table of functions and names */

/* Commands that take no arguments */
s_fn_table fn_table[] =
{
	{"listsel-test", listsel_test},

	/* moving around */
	{"what-cursor-position", what_cursor_position},

	/* editing */

	/* block commands */
	{"preview-block", preview_block},
	{"print-block", print_block},

	/* new window */
	{"delete-window", delete_window},
	{"delete-other-windows", delete_other_windows},
	{"split-window-vertically", split_window_vertically},
	{"other-window", other_window},

	/* buffers and windows */
	{"switch-to-buffer", switch_to_buffer},
	{"kill-buffer", kill_buffer},
	{"load-buffer", load_buffer},
	{"save-buffer", save_buffer},
	{"save-buffer-as", save_buffer_as},
	{"load-external", load_external},
	{"save-external", save_external},

	/* help commands */
	{"print-version", print_version},

	{"quit-egon", quit_egon},
	/* low level functions */
	{"bell", bell},

	{"ani-begin", ani_begin},
	{"ani-end", ani_end},
	{"del-object", del_object},
	{"del-time", del_time},
	{"ani-c-test", ani_c_test},
	{"get-format", get_format},
	{(char *) 0, (LISP(*)())0}
};

/* Commands that take 1 argument */

s_fn_table fn_table1[] = {
	/* low level functions */
	{"spawn", spawn},
	{"execute-interpreter-command", execute_interpreter_command},
	{"buffer-changed", buffer_changed},
	{"execute", lexecute},
	{"input-warp-pointer", linput_warp_pointer},

	{"ani-object", lani_object},
	{"ani-background", ani_background},
	{"ani-time", ani_time},
	{"ani-duration", lani_duration},
	{"ani-delta", lani_delta},
	{"select-object", select_object},
	{"select-tick", select_tick},
	{"set-type", set_type},
	{"set-format", set_format},
	{NULL, NULL}
};

/* Commands that take 2 arguments */
s_fn_table fn_table2[] = {
	{"ask-for-str", lask_for_str},
	{"alertbox", lalertbox},

	{"ani-property", ani_property},
	{"ani-geometry", lani_geometry},
	{"ani-ctl", lani_ctl},
	{NULL, NULL}
};

void init_cmds()
{
	int i;

	for (i = 0; fn_table[i].name; i++)
		init_subr_0(fn_table[i].name, fn_table[i].function);
	for (i = 0; fn_table1[i].name; i++)
		init_subr_1(fn_table1[i].name, fn_table1[i].function);
	for (i = 0; fn_table2[i].name; i++)
		init_subr_2(fn_table2[i].name, fn_table2[i].function);
}

