/*
   Egon Animator
   Copyright (C) 1997  Ulric Eriksson <ulric@edu.stockholm.se>

   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2, or (at your option)
   any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */

/*
 * matrix.c
 *
 * This module hides the details of stringpool and matrix handling.
 */

#include <stdio.h>
#include <string.h>
#include <stdlib.h>
#include <ctype.h>
#include <X11/Xlib.h>
#include <X11/Intrinsic.h>
#include <X11/StringDefs.h>
#include <X11/xpm.h>

#include "animator.h"
#include "egon.h"
#include "types.h"
#include "../common/fonts.h"
#include "../xcommon/embed.h"
#include "../common/cmalloc.h"

int max_lines = BUFFER_ROWS;
int max_columns = BUFFER_COLS;


style styles[16] = {
	{"Default", TIMES | SIZE_12, 20, STY_DEFAULT},
	{"Header 1", HELVETICA | SIZE_24 | BOLD, 30, STY_DEFAULT},
	{"Header 2", HELVETICA | SIZE_18 | BOLD, 20, STY_DEFAULT},
	{"Header 3", HELVETICA | SIZE_14 | BOLD, 20, STY_DEFAULT},
	{"Header 4", HELVETICA | SIZE_12 | BOLD, 20, STY_DEFAULT},
	{"Header 5", HELVETICA | SIZE_10 | BOLD, 20, STY_DEFAULT},
	{"Header 6", HELVETICA | SIZE_8 | BOLD, 20, STY_DEFAULT},
	{"Address", TIMES | SIZE_12 | ITALIC, 20, STY_ADDRESS},
	{"Title", TIMES | SIZE_12 | BOLD, 20, STY_TITLE},
	{"Abstract", TIMES | SIZE_14 | BOLD, 20, STY_ABSTRACT},
	{"Preformatted", COURIER | SIZE_12, 20, STY_PREFORMAT},
	{"User 1", TIMES | SIZE_12, 20, STY_USER1},
	{"User 2", TIMES | SIZE_12, 20, STY_USER2},
	{"User 3", TIMES | SIZE_12, 20, STY_USER3},
	{"User 4", TIMES | SIZE_12, 20, STY_USER4},
	{"Embed", TIMES | SIZE_12, 20, STY_DEFAULT}
};


char *fmt_put(char *fmt, int mode)
{
	char *p;

	if (mode > STY_EMBED)	/* old file, adjust */
		mode >>= FMT_SHIFT;

	p = strtok(fmt, ",\n");
	if (!p) return NULL;
	styles[mode].name = cstrdup(p);
	p = strtok(NULL, ",\n");
	if (!p) return NULL;
	styles[mode].format = atol(p);
	p = strtok(NULL, ",\n");
	if (!p) return NULL;
	styles[mode].height = atoi(p);
	p = strtok(NULL, ",\n");
	if (!p) return NULL;
	styles[mode].follower = atoi(p);
	return fmt;
}

char *fmt_get(char *fmt, int mode)
{
	static char buf[80];

/*	mode >>= FMT_SHIFT;
*/
	if (fmt == NULL) {
		fmt = buf;
	}

	sprintf(fmt, "%s,%ld,%d,%d", styles[mode].name,
		styles[mode].format, styles[mode].height, styles[mode].follower);
	return fmt;
}

/*X
   spread **new_matrix()
   Creates new empty matrix.  Returns NULL if it failed.
   X */
ani_object *new_matrix()
{
	/* well, it can't get any more empty than NULL */
	return NULL;
}

/*X
   void free_matrix(spread **matrix)
   Frees the memory used by the matrix.
   X */
void free_matrix(ani_object *matrix)
{
	/* It would be nice if this actually did something */
}

ani_object *last_object(buffer *b)
{
	ani_object *o;
	if (!b) return NULL;
	o = b->cast;
	if (!o) return o;
	while (o->next) o = o->next;
	return o;
}

ani_script *last_script(ani_object *o)
{
	ani_script *s;
	if (!o) return NULL;
	s = o->script;
	if (!s) return s;
	while (s->next) s = s->next;
	return s;
}

int ins_format(buffer *buf, ani_object *o, ani_script *s, int format)
{
	if (!buf || !o || !s) return 0;

	s->font = format & 127;
	s->color = (format & COLOR_MASK) >> COLOR_SHIFT;
	return 1;
}

int ret_format(buffer *buf, ani_object *o, ani_script *s)
{
	if (!buf || !o || !s)
		return HELVETICA | SIZE_14;

	return s->font | (s->color << COLOR_SHIFT);
}

int ret_font(buffer *buf, ani_object *o, ani_script *s)
{
	return ret_format(buf, o, s) & 127;
}

